package com.cim.idm.wmsextend.generic;

import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * cron表达式格式：<br>
 * (cron = "* * * * * *") <br>
 * {秒数} {分钟} {小时} {日期} {月份} {星期} {年份(可为空)}<br>
 * cron在线生成器 http://cron.qqe2.com/ <br>
 * <br>
 * 字段 允许值 允许的特殊字符<br>
 * 秒 0-59 , - * /<br>
 * 分 0-59 , - * /<br>
 * 小时 0-23 , - * /<br>
 * 日期 1-31 , - * ? / L W C<br>
 * 月份 1-12 或者 JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV , DEC, - * /<br>
 * 星期 1-7 （1＝星期日） 或者 SUN, MON, TUE, WED, THU, FRI , SAT , - * ? / L C #<br>
 * 年（可选） 留空, 1970-2099 , - * /<br>
 *<br>
 * 每个符号的意义<br>
 * * 表示所有值；<br>
 * ? 表示未说明的值，即不关心它为何值；<br>
 * - 表示一个指定的范围；<br>
 * , 表示附加一个可能值；<br>
 * / 符号前表示开始时间，符号后表示每次递增的值；<br>
 * L("last") ("last") "L" 用在day-of-month字段意思是 "这个月最后一天"；用在 day-of-week字段, 它简单意思是 "7" or "SAT"。 如果在day-of-week字段里和数字联合使用，它的意思就是 "这个月的最后一个星期几" – 例如： "6L" means "这个月的最后一个星期五". 当我们用“L”时，不指明一个列表值或者范围是很重要的，不然的话，我们会得到一些意想不到的结果。<br>
 * W("weekday") 只能用在day-of-month字段。用来描叙最接近指定天的工作日（周一到周五）。例如：在day-of-month字段用“15W”指“最接近这个 月第15天的工作日”，即如果这个月第15天是周六，那么触发器将会在这个月第14天即周五触发；如果这个月第15天是周日，那么触发器将会在这个月第 16天即周一触发；如果这个月第15天是周二，那么就在触发器这天触发。注意一点：这个用法只会在当前月计算值，不会越过当前月。“W”字符仅能在 day-of-month指明一天，不能是一个范围或列表。也可以用“LW”来指定这个月的最后一个工作日。<br>
 * # 只能用在day-of-week字段。用来指定这个月的第几个周几。例：在day-of-week字段用"6#3"指这个月第3个周五（6指周五，3指第3个）。如果指定的日期不存在，触发器就不会触发。<br>
 * C 指和calendar联系后计算过的值。例：在day-of-month 字段用“5C”指在这个月第5天或之后包括calendar的第一天；在day-of-week字段用“1C”指在这周日或之后包括calendar的第一天。<br>
 *<br>
 * 一些cron表达式案例<br>
 * &quot;0/5 * * * * ?&quot; 每隔5秒执行一次<br>
 * &quot;0 0/1 * * * ?&quot; 每隔1分钟执行一次<br>
 * &quot;0 0 5-15 * * ?&quot; 每天5-15点整点触发<br>
 * &quot;0 0/3 * * * ?&quot; 每三分钟触发一次<br>
 * &quot;0 0-5 14 * * ?&quot; 在每天下午2点到下午2:05期间的每1分钟触发 <br>
 * &quot;0 0/5 14 * * ?&quot; 在每天下午2点到下午2:55期间的每5分钟触发<br>
 * &quot;0 0/5 14,18 * * ?&quot; 在每天下午2点到2:55期间和下午6点到6:55期间的每5分钟触发<br>
 * &quot;0 0/30 9-17 * * ?&quot; 朝九晚五工作时间内每半小时<br>
 * &quot;0 0 10,14,16 * * ?&quot; 每天上午10点，下午2点，4点 <br>
 *
 * &quot;0 0 12 ? * WED&quot; 表示每个星期三中午12点<br>
 * &quot;0 0 17 ? * TUES,THUR,SAT&quot; 每周二、四、六下午五点<br>
 * &quot;0 10,44 14 ? 3 WED&quot; 每年三月的星期三的下午2:10和2:44触发<br>
 * &quot;0 15 10 ? *&quot; MON-FRI 周一至周五的上午10:15触发<br>
 * &quot;0 0 23 L * ?&quot; 每月最后一天23点执行一次<br>
 * &quot;0 15 10 L * ?&quot; 每月最后一日的上午10:15触发<br>
 * &quot;0 15 10 ? * 6L&quot; 每月的最后一个星期五上午10:15触发<br>
 * &quot;0 15 10 * * ?&quot;2005 2005年的每天上午10:15触发<br>
 * &quot;0 15 10 ? * 6L&quot; 2002-2005 2002年至2005年的每月的最后一个星期五上午10:15触发<br>
 * &quot;0 15 10 ? * 6#3&quot; 每月的第三个星期五上午10:15触发<br>
 *<br>
 * "30 * * * * ?" 每半分钟触发任务<br>
 * "30 10 * * * ?" 每小时的10分30秒触发任务<br>
 * "30 10 1 * * ?" 每天1点10分30秒触发任务<br>
 * "30 10 1 20 * ?" 每月20号1点10分30秒触发任务<br>
 * "30 10 1 20 10 ? *" 每年10月20号1点10分30秒触发任务<br>
 * "30 10 1 20 10 ? 2011" 2011年10月20号1点10分30秒触发任务<br>
 * "30 10 1 ? 10 * 2011" 2011年10月每天1点10分30秒触发任务<br>
 * "30 10 1 ? 10 SUN 2011" 2011年10月每周日1点10分30秒触发任务<br>
 * "15,30,45 * * * * ?" 每15秒，30秒，45秒时触发任务<br>
 * "15-45 * * * * ?" 15到45秒内，每秒都触发任务<br>
 * "15/5 * * * * ?" 每分钟的每15秒开始触发，每隔5秒触发一次<br>
 * "15-30/5 * * * * ?" 每分钟的15秒到30秒之间开始触发，每隔5秒触发一次<br>
 * "0 0/3 * * * ?" 每小时的第0分0秒开始，每三分钟触发一次<br>
 * "0 15 10 ? * MON-FRI" 星期一到星期五的10点15分0秒触发任务<br>
 * "0 15 10 L * ?" 每个月最后一天的10点15分0秒触发任务<br>
 * "0 15 10 LW * ?" 每个月最后一个工作日的10点15分0秒触发任务<br>
 * "0 15 10 ? * 5L" 每个月最后一个星期四的10点15分0秒触发任务<br>
 * "0 15 10 ? * 5#3" 每个月第三周的星期四的10点15分0秒触发任务<br>
 *
 */
@Component
public class MyScheduler {

    @Scheduled(cron = "0 0/2 * * * ?")
    public void test(){
        System.out.println("My test timer:" + new Date());
    }
}
